"""Top-level package for containers-sugar."""


__author__ = 'Ivan Ogasawara'
__email__ = 'ivan.ogasawara@gmail.com'
__version__ = '1.4.4'  # semantic-release


from containers_sugar.sugar import Sugar  # noqa: F401
